@@ -901,7 +901,7 @@ alp_GetFrameTime(alpobject *self, PyObject *args)
 		Py_XDECREF(v1);
 		return NULL;
 	}
-	ret = Py_BuildValue("(OO)", v0, v1);
+	ret = PyTuple_Pack(2, v0, v1);
 	Py_DECREF(v0);
 	Py_DECREF(v1);
 	return ret;
@@ -1482,7 +1482,8 @@ al_GetParams(PyObject *self, PyObject *args)
 	}
 	if (alGetParams(resource, pvs, npvs) < 0)
 		goto error;
-	v = PyList_New(npvs);
+	if (!(v = PyList_New(npvs)))
+		goto error;
 	for (i = 0; i < npvs; i++) {
 		if (pvs[i].sizeOut < 0) {
 			char buf[32];
@@ -1632,9 +1633,11 @@ al_QueryValues(PyObject *self, PyObject *args)
 	if (nvals < 0)
 		goto cleanup;
 	if (nvals > setsize) {
+		ALvalue *old_return_set = return_set;
 		setsize = nvals;
 		PyMem_RESIZE(return_set, ALvalue, setsize);
 		if (return_set == NULL) {
+			return_set = old_return_set;
 			PyErr_NoMemory();
 			goto cleanup;
 		}
@@ -1692,6 +1695,7 @@ al_GetParamInfo(PyObject *self, PyObject *args)
 	if (alGetParamInfo(res, param, &pinfo) < 0)
 		return NULL;
 	v = PyDict_New();
+	if (!v) return NULL;
 
 	item = PyInt_FromLong((long) pinfo.resource);
 	PyDict_SetItemString(v, "resource", item);
@@ -1996,6 +2000,8 @@ inital(void)
 	m = Py_InitModule4("al", al_methods,
 		al_module_documentation,
 		(PyObject*)NULL,PYTHON_API_VERSION);
+	if (m == NULL)
+		return;
 
 	/* Add some symbolic constants to the module */
 	d = PyModule_GetDict(m);
